/*-----------------------------------------------------------------------*/
/*  (C) Copyright Institute of Astronomy ETHZ 8092 Zuerich Switzerland   */
/*-----------------------------------------------------------------------*/
/*  Programmname: rxrs232.cpp                                            */
/*                                                                       */
/*  Revision: V1.0     Date: 26.11.2003    Autor: Chr. Monstein          */
/*                                                                       */
/*  Purpose: Receiver control unit Radiospectrometer CALLISTO            */
/*                                                                       */
/*  Compiler: BORLAND C++Builder 6                                       */
/*-----------------------------------------------------------------------*/

// Created by: Chr. Monstein 05.05.2003
// Updated by: Chr. Monstein 17.06.2003
// Updated by: Chr. Monstein 26.11.2004 programmable baudrate, integration deleted
// Updated by: Chr. Monstein 29.12.2010 more COMs
// Updated by: Chr. Monstein 17.06.2011 more error tolerant

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#define WIN32

#include <stdio.h> // sprintf

#include "WSC.H"
#include "RXRS232.h"
#include "mainunit.h"

#pragma package(smart_init)
//---------------------------------------------------------------------------

TWrite *RX_SerialThread;
int RX_Port; // Receiver control unit RCU
bool comerr=true;

Byte RX_ringbuff[RXbuflen];
long RX_in=0, RX_out=0;
//---------------------------------------------------------------------------
__fastcall TWrite::TWrite(bool CreateSuspended) : TThread(CreateSuspended)
{
}
//---------------------------------------------------------------------------
void __fastcall TWrite::Init(char *rxcomport)
{
    char Temp[80];
    int Code;
    if (strstr(rxcomport,"COM1" )) RX_Port=COM1;
    if (strstr(rxcomport,"COM2" )) RX_Port=COM2;
    if (strstr(rxcomport,"COM3" )) RX_Port=COM3;
    if (strstr(rxcomport,"COM4" )) RX_Port=COM4;
    if (strstr(rxcomport,"COM5" )) RX_Port=COM5;
    if (strstr(rxcomport,"COM6" )) RX_Port=COM6;
    if (strstr(rxcomport,"COM7" )) RX_Port=COM7;
    if (strstr(rxcomport,"COM8" )) RX_Port=COM8;
    if (strstr(rxcomport,"COM9" )) RX_Port=COM9;
    if (strstr(rxcomport,"COM10")) RX_Port=COM10;
    if (strstr(rxcomport,"COM11")) RX_Port=COM11;
    if (strstr(rxcomport,"COM12")) RX_Port=COM12;

    // Code = SioDebug('R'); // power USB-device
    Code = SioReset(RX_Port,8192,1024); // COM, RX-buffer, TX-buffer
    if(Code<0)
    {
        SioWinError((LPSTR)Temp,80);
        Application->MessageBox("Port does not exist", "Communication error", MB_OK);
        ieee->Button2Click(0);
        comerr = true;
    }
    else
    {
        SioParms(RX_Port,WSC_NoParity,WSC_OneStopBit,WSC_WordLength8);
        SioBaud(RX_Port,WSC_Baud115200);
        SioDTR(RX_Port,'S');
        SioRTS(RX_Port,'S');
        comerr = false;
    }
}
//---------------------------------------------------------------------------
void __fastcall TWrite::Transmit(char byte) // Send byte to client via RS
{
    SioPutc(RX_Port, byte);
}
//---------------------------------------------------------------------------
void __fastcall TWrite::Send(char * cmd) // Send byte to client via RS
{
    SioPuts(RX_Port, (LPSTR)cmd, strlen(cmd)); // string to receiver
}
//---------------------------------------------------------------------------
void __fastcall TWrite::GetRXRS232Data(void) // Private activities
{
    int cnt;
    char buffer[512]; // we expect 4KByte/sec = 80Byte/20msec
    cnt = SioGets(RX_Port,(LPSTR)buffer,512);  // (LPSTR) = mit null beendeter String
    for (int i=0; i<cnt; i++) // fill up FPU ringbuffer
    {
        RX_ringbuff[RX_in] = buffer[i];
        RX_in = (RX_in+1)%RXbuflen;
    }
}
//---------------------------------------------------------------------------
void __fastcall TWrite::Execute() // Main job extra-thread
{
    while(1) // almost forever...
    {
        if(Terminated) return; // exit thread
        GetRXRS232Data();
        Sleep(2); // wait
    }
}
//---------------------------------------------------------------------------
void __fastcall TWrite::Close() // termination
{
    SioDone(RX_Port);
    Terminate(); //RX_SerialThread->Terminate();
    WaitFor(); //RX_SerialThread->WaitFor();
}
//---------------------------------------------------------------------------
