/*-----------------------------------------------------------------------*/
/*  (C) Copyright Institute of Astronomy ETHZ 8092 Zuerich Switzerland   */
/*-----------------------------------------------------------------------*/
/*  Programmname: mainunit.cpp                                           */
/*                                                                       */
/*  Revision: V1.0     Date: 05.05.2003    Autor: Chr. Monstein          */
/*                                                                       */
/*  Purpose: Receiver control unit Radiospectrometer CALLISTO            */
/*                                                                       */
/*  Compiler: BORLAND C++Builder 6                                       */
/*-----------------------------------------------------------------------*/

// Created by: Chr. Monstein 05.05.2003
// Updated by: Chr. Monstein 17.06.2003
// Updated by: Chr. Monstein 26.11.2004
// Updated by: Chr. Monstein 29.12.2010 adaptions to baboon version
// Updated by: Chr. Monstein 17.06.2011 more error-tolerant
//---------------------------------------------------------------------------

#pragma hdrstop
#include <vcl.h>

#include "mainunit.h"
#include <stdio.h>

#include "WSC.H"
#include "RXRS232.H"
#include <windows.h>
#include <math.h>

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"

//---------------------------------------------------------------------------

Tieee *ieee;

// import form RXRS232.cpp
extern TWrite *RX_SerialThread; // RX activities
extern Byte RX_ringbuff[RXbuflen];
extern long RX_in, RX_out;
extern bool comerr;
//---------------------------------------------------------------------------
__fastcall Tieee::Tieee(TComponent* Owner) : TForm(Owner)
{
    RX_SerialThread = new TWrite(false);
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Button1Click(TObject *Sender) // connect
{
    RX_SerialThread->Init(Edit1->Text.c_str()); // Connect RX-interface Port
    Sleep(100);

    if (!comerr)
    {
      Edit2Change(0); // Set default frequency
      Edit3Change(0); // Set default gain control voltage
      Edit4Change(0); // Set default time base

      RX_SerialThread->Send("%2\r"); // set fixed format mV
      Sleep(10);

      Timer1->Enabled = true;  // Start
    }
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Button2Click(TObject *Sender) // stop
{
    Timer1->Enabled = false; // Stop
    Sleep(10);
    RX_SerialThread->Close();
}
//---------------------------------------------------------------------------
__fastcall Tieee::~Tieee()  // destructor
{
    //RX_SerialThread->Close();
    //delete RX_SerialThread;
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Timer1Timer(TObject *Sender) // state machine
{
    short status;
    char st[80], data[240];
    static int state = 0;
    char ch;
    static int v1;

    switch(state) // execute every x msec
    {
        case 0: // ask for ADC
        {
            RX_SerialThread->Send("A0\r"); // detector voltage
            state++;
            break;
        }

        case 1: // wait for incoming data
        {
            int i = 0;
            do
            {
                ch = RX_ringbuff[RX_out];
                RX_out = (RX_out+1)%RXbuflen;
                if (ch==13) break;
                data[i++]=ch;
                if (i>238) i=0;
            } while (1);
            data[i]='\0';

            int p=0;
            if (data[0]=='$')
              p = sscanf(data,"$CRX:ADC0=%u",&v1);
            if (p==1)
            {
                sprintf(st,"%umV",v1);
                Label2->Caption = st;  // voltage
                int pp = v1/5; // reduction due to limited max of PB
                ProgressBar1->Position = pp;
                int tp = 500 - TrackBar1->Position;
                if (pp > tp)
                {
                   Label12->Caption="rf > trigger";
                   Label12->Color = clLime;
                   Beep();
                }
                else
                {
                   Label12->Caption="rf < trigger";
                   Label12->Color = clRed;
                }
                //Label13->Caption= 5*tp;
                state = 0; // everything again ...
            }
            break;
        }
    } // end switch
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Edit2Change(TObject *Sender)
{
   float freq = atof(Edit2->Text.c_str());
   if (freq <  45.0) freq =  45.0;
   if (freq > 870.0) freq = 870.0;
   char st[40];
   sprintf(st,"F0%f\r",freq); // frequency control
   RX_SerialThread->Send(st);
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Edit3Change(TObject *Sender)
{
    char st[80];
    int agc = atoi(Edit3->Text.c_str());
    if (agc <   0) agc =   0;
    if (agc > 255) agc = 255;
    sprintf(st,"O%3i\r",agc); // gain control
    RX_SerialThread->Send(st);
}
//---------------------------------------------------------------------------
void __fastcall Tieee::Edit4Change(TObject *Sender)
{
    char st[80];
    int dt = atoi(Edit4->Text.c_str());
    if (dt < 20) dt = 20;
    if (dt > 10000) dt = 10000;
    Timer1->Interval = dt/2; // 1/2 due to state machine
}
//---------------------------------------------------------------------------
void __fastcall Tieee::FormClose(TObject *Sender, TCloseAction &Action)
{
    RX_SerialThread->Close();
    //delete RX_SerialThread;
    Application->Terminate();
}
//---------------------------------------------------------------------------

